//
//  CipherText.c
//  study
//
//  Created by 國居 貴浩 on 12/01/14.
//  Copyright (c) 2012年 TETERA. All rights reserved.
//

#include <stdio.h>
#include <stdlib.h>
#include "CipherText.h"

int stringlen(const char* text);

//  plane_textをshift分ずらしで暗号化する。
//  暗号化された文字列はcipher.textに入る。
//  失敗すると0、成功すると1を戻す。
static int cipher(const char * plain_text, CipherText * ioCipher)
{
    int length = stringlen(plain_text);
    if (ioCipher->text != NULL) {        // 以前に確保した領域があるので、先に解放する。
        free(ioCipher->text);
    }
    ioCipher->text = malloc(sizeof(char) * (length + 1));   //  新しく領域確保。
    if (ioCipher->text == NULL) {       //  確保に失敗。
        return 0;  //   失敗
    }
    int shift[2];
    shift[0] = ioCipher->altShift;
    shift[1] = ioCipher->shift;
    for (int i = 0; i < length; i++) {                  //  lengthが全文字数。
        ioCipher->text[i] = *plain_text + shift[i % 2];    	// 1文字暗号化する。
        plain_text++;                           		// 次の文字にすすめる。
    }
    ioCipher->text[length] = 0;                         //  終端を設定。
    return 1;   //  成功
}

//  cipherに設定されている暗号文字列を解凍する。
//  解凍に成功すると、解凍された文字列の先頭番地を戻し、失敗するとNULLを戻す。
static char* decipher(const CipherText * cipher)
{
    int length = stringlen(cipher->text);
    char* text = malloc(sizeof(char) * (length + 1));   // 新しく領域確保。
    if (text == NULL) {
        return NULL;    //  失敗
    }
    // 暗号を解く
    int shift[2];
    shift[0] = cipher->altShift;
    shift[1] = cipher->shift;
    for (int i = 0; i < length; i++) {                  //  lengthが全文字数。
        text[i] = cipher->text[i] - shift[i % 2];      // 1文字、暗号を解く。
    }
    text[length] = 0;    //  終端を設定。
    return text;   //  成功
}


//  初期化
void initCipherDoubleText(CipherText* ioCipher, int shift, int altShift)
{
    ioCipher->shift = shift;            //  ずらし量を保存
    ioCipher->altShift = altShift;      //  ずらし量2を保存
    ioCipher->text = NULL;
    ioCipher->cipher = cipher;
    ioCipher->decipher = decipher;
}


